@@ -1,20 +1,21 @@
 # -*- coding: utf-8 -*-
-from __future__ import with_statement
+from collections import OrderedDict
+from importlib import import_module
 
 from django.conf import settings
-from django.conf.urls import patterns
-from django.contrib.sites.models import Site
 from django.core.exceptions import ImproperlyConfigured
 from django.core.urlresolvers import (RegexURLResolver, Resolver404, reverse,
-    RegexURLPattern)
+                                      RegexURLPattern)
+from django.db import OperationalError, ProgrammingError
 from django.utils import six
-from django.utils.importlib import import_module
-from django.utils.translation import get_language
+from django.utils.translation import get_language, override
 
 from cms.apphook_pool import apphook_pool
 from cms.models.pagemodel import Page
-from cms.utils.i18n import force_language, get_language_list
-
+from cms.utils import get_current_site
+from cms.utils.compat import DJANGO_1_8, DJANGO_1_9
+from cms.utils.i18n import get_language_list
+from cms.utils.moderator import use_draft
 
 APP_RESOLVERS = []
 
@@ -38,6 +39,9 @@ def applications_page_check(request, current_page=None, path=None):
     for lang in get_language_list():
         if path.startswith(lang + "/"):
             path = path[len(lang + "/"):]
+
+    use_public = not use_draft(request)
+
     for resolver in APP_RESOLVERS:
         try:
             page_id = resolver.resolve_page_id(path)
@@ -46,10 +50,12 @@ def applications_page_check(request, current_page=None, path=None):
             # If current page was matched, then we have some override for
             # content from cms, but keep current page. Otherwise return page
             # to which was application assigned.
-            return page
+            return page if use_public else page.publisher_public
         except Resolver404:
             # Raised if the page is not managed by an apphook
             pass
+        except Page.DoesNotExist:
+            pass
     return None
 
 
@@ -93,12 +99,13 @@ class AppRegexURLResolver(RegexURLResolver):
                             tried.extend([pattern])
                     else:
                         if sub_match:
-                            return pattern.page_id
+                            return getattr(pattern, 'page_id', None)
                         tried.append(pattern.regex.pattern)
             raise Resolver404({'tried': tried, 'path': new_path})
 
 
-def recurse_patterns(path, pattern_list, page_id, default_args=None, nested=False):
+def recurse_patterns(path, pattern_list, page_id, default_args=None,
+                     nested=False):
     """
     Recurse over a list of to-be-hooked patterns for a given path prefix
     """
@@ -110,16 +117,22 @@ def recurse_patterns(path, pattern_list, page_id, default_args=None, nested=Fals
         path = path.lstrip('^')
         regex = r'^%s%s' % (path, app_pat) if not nested else r'^%s' % (app_pat)
         if isinstance(pattern, RegexURLResolver):
-            # this is an 'include', recurse!
-            resolver = RegexURLResolver(regex, 'cms_appresolver',
-                                        pattern.default_kwargs, pattern.app_name, pattern.namespace)
-            resolver.page_id = page_id
             # include default_args
             args = pattern.default_kwargs
             if default_args:
                 args.update(default_args)
-            # see lines 243 and 236 of urlresolvers.py to understand the next line
-            resolver._urlconf_module = recurse_patterns(regex, pattern.url_patterns, page_id, args, nested=True)
+            if DJANGO_1_8:
+                # this is an 'include', recurse!
+                resolver = RegexURLResolver(regex, 'cms_appresolver',
+                                            pattern.default_kwargs, pattern.app_name, pattern.namespace)
+                # see lines 243 and 236 of urlresolvers.py to understand the next line
+                resolver._urlconf_module = recurse_patterns(regex, pattern.url_patterns, page_id, args, nested=True)
+            else:
+                # see lines 243 and 236 of urlresolvers.py to understand the next line
+                urlconf_module = recurse_patterns(regex, pattern.url_patterns, page_id, args, nested=True)
+                # this is an 'include', recurse!
+                resolver = RegexURLResolver(regex, urlconf_module,
+                                            pattern.default_kwargs, pattern.app_name, pattern.namespace)
         else:
             # Re-do the RegexURLPattern with the new regular expression
             args = pattern.default_args
@@ -127,7 +140,7 @@ def recurse_patterns(path, pattern_list, page_id, default_args=None, nested=Fals
                 args.update(default_args)
             resolver = RegexURLPattern(regex, pattern.callback,
                                        args, pattern.name)
-            resolver.page_id = page_id
+        resolver.page_id = page_id
         newpatterns.append(resolver)
     return newpatterns
 
@@ -140,7 +153,10 @@ def _set_permissions(patterns, exclude_permissions):
             _set_permissions(pattern.url_patterns, exclude_permissions)
         else:
             from cms.utils.decorators import cms_perms
-            pattern._callback = cms_perms(pattern.callback)
+            if DJANGO_1_9:
+                pattern._callback = cms_perms(pattern.callback)
+            else:
+                pattern.callback = cms_perms(pattern.callback)
 
 
 def get_app_urls(urls):
@@ -151,8 +167,10 @@ def get_app_urls(urls):
                 raise ImproperlyConfigured(
                     "URLConf `%s` has no urlpatterns attribute" % urlconf)
             yield getattr(mod, 'urlpatterns')
-        else:
+        elif isinstance(urlconf, (list, tuple)):
             yield urlconf
+        else:
+            yield [urlconf]
 
 
 def get_patterns_for_title(path, title):
@@ -162,7 +180,7 @@ def get_patterns_for_title(path, title):
     """
     app = apphook_pool.get_apphook(title.page.application_urls)
     url_patterns = []
-    for pattern_list in get_app_urls(app.urls):
+    for pattern_list in get_app_urls(app.get_urls(title.page, title.language)):
         if path and not path.endswith('/'):
             path += '/'
         page_id = title.page.id
@@ -171,66 +189,83 @@ def get_patterns_for_title(path, title):
 
 
 def get_app_patterns():
+    try:
+        site = get_current_site()
+        return _get_app_patterns(site)
+    except (OperationalError, ProgrammingError):
+        # ignore if DB is not ready
+        # Starting with Django 1.9 this code gets called even when creating
+        # or running migrations. So in many cases the DB will not be ready yet.
+        return []
+
+
+def _get_app_patterns(site):
     """
     Get a list of patterns for all hooked apps.
 
     How this works:
 
-    By looking through all titles with an app hook (application_urls) we find all
-    urlconf modules we have to hook into titles.
+    By looking through all titles with an app hook (application_urls) we find
+    all urlconf modules we have to hook into titles.
 
     If we use the ML URL Middleware, we namespace those patterns with the title
     language.
 
     All 'normal' patterns from the urlconf get re-written by prefixing them with
     the title path and then included into the cms url patterns.
+
+    If the app is still configured, but is no longer installed/available, then
+    this method returns a degenerate patterns object: patterns('')
     """
     from cms.models import Title
 
-    try:
-        current_site = Site.objects.get_current()
-    except Site.DoesNotExist:
-        current_site = None
     included = []
 
     # we don't have a request here so get_page_queryset() can't be used,
     # so use public() queryset.
     # This can be done because url patterns are used just in frontend
+    title_qs = Title.objects.public().filter(page__node__site=site)
 
-    title_qs = Title.objects.public().filter(page__site=current_site)
-
-    hooked_applications = {}
+    hooked_applications = OrderedDict()
 
     # Loop over all titles with an application hooked to them
-    for title in title_qs.exclude(page__application_urls=None).exclude(page__application_urls='').select_related():
+    titles = (title_qs.exclude(page__application_urls=None)
+              .exclude(page__application_urls='')
+              .order_by('-page__node__path').select_related())
+    # TODO: Need to be fixed for django-treebeard when forward ported to 3.1
+    for title in titles:
         path = title.path
-        mix_id = "%s:%s:%s" % (path + "/", title.page.application_urls, title.language)
+        mix_id = "%s:%s:%s" % (
+            path + "/", title.page.application_urls, title.language)
         if mix_id in included:
             # don't add the same thing twice
             continue
         if not settings.APPEND_SLASH:
             path += '/'
+        app = apphook_pool.get_apphook(title.page.application_urls)
+        if not app:
+            continue
         if title.page_id not in hooked_applications:
             hooked_applications[title.page_id] = {}
-        app = apphook_pool.get_apphook(title.page.application_urls)
         app_ns = app.app_name, title.page.application_namespace
-        with force_language(title.language):
-            hooked_applications[title.page_id][title.language] = (app_ns, get_patterns_for_title(path, title), app)
+        with override(title.language):
+            hooked_applications[title.page_id][title.language] = (
+                app_ns, get_patterns_for_title(path, title), app)
         included.append(mix_id)
         # Build the app patterns to be included in the cms urlconfs
     app_patterns = []
     for page_id in hooked_applications.keys():
         resolver = None
         for lang in hooked_applications[page_id].keys():
-            (app_ns, inst_ns), current_patterns, app = hooked_applications[page_id][lang]
+            (app_ns, inst_ns), current_patterns, app = hooked_applications[page_id][lang]  # nopyflakes
             if not resolver:
-                resolver = AppRegexURLResolver(r'', 'app_resolver', app_name=app_ns, namespace=inst_ns)
+                resolver = AppRegexURLResolver(
+                    r'', 'app_resolver', app_name=app_ns, namespace=inst_ns)
                 resolver.page_id = page_id
             if app.permissions:
                 _set_permissions(current_patterns, app.exclude_permissions)
 
-            extra_patterns = patterns('', *current_patterns)
-            resolver.url_patterns_dict[lang] = extra_patterns
+            resolver.url_patterns_dict[lang] = current_patterns
         app_patterns.append(resolver)
         APP_RESOLVERS.append(resolver)
     return app_patterns
