@@ -1,15 +1,21 @@
 # -*- coding: utf-8 -*-
 from cms.models import CMSPlugin
 from cms.models.fields import PlaceholderField
-from cms.utils.compat.dj import python_2_unicode_compatible
 from cms.utils.copy_plugins import copy_plugins_to
 from django.db import models
+from django.utils.encoding import python_2_unicode_compatible
+
 
 @python_2_unicode_compatible
 class PlaceholderReference(CMSPlugin):
+    cmsplugin_ptr = models.OneToOneField(
+        CMSPlugin,
+        on_delete=models.CASCADE,
+        related_name='cms_placeholderreference',
+        parent_link=True,
+    )
     name = models.CharField(max_length=255)
     placeholder_ref = PlaceholderField(slotname='clipboard')
-
     class Meta:
         app_label = 'cms'
 
@@ -20,7 +26,8 @@ class PlaceholderReference(CMSPlugin):
         copy_plugins_to(self.placeholder_ref.get_plugins(), placeholder, to_language=language)
 
     def copy_from(self, placeholder, language):
-        copy_plugins_to(placeholder.get_plugins(language), self.placeholder_ref, to_language=self.language)
+        plugins = placeholder.get_plugins(language)
+        return copy_plugins_to(plugins, self.placeholder_ref, to_language=self.language)
 
     def move_to(self, placeholder, language):
         for plugin in self.placeholder_ref.get_plugins():
@@ -32,4 +39,4 @@ class PlaceholderReference(CMSPlugin):
         for plugin in placeholder.get_plugins():
             plugin.placeholder = self.placeholder_ref
             plugin.language = language
-            plugin.save()
\ No newline at end of file
+            plugin.save()
