@@ -1,7 +1,6 @@
 # -*- coding: utf-8 -*-
 from contextlib import contextmanager
 import datetime
-import json
 import pickle
 import warnings
 
@@ -15,15 +14,14 @@ from django.contrib.admin.widgets import FilteredSelectMultiple, RelatedFieldWid
 from django.core import urlresolvers
 from django.core.exceptions import ImproperlyConfigured
 from django.forms.widgets import Media
-from sekizai.context import SekizaiContext
 from django.test.testcases import TestCase
 from django.utils import timezone
 from django.utils.encoding import force_text
+from django.utils.translation import override as force_language
 
 from cms import api
-from cms.constants import PLUGIN_MOVE_ACTION, PLUGIN_COPY_ACTION
 from cms.exceptions import PluginAlreadyRegistered, PluginNotRegistered, DontUsePageAttributeWarning
-from cms.models import Page, Placeholder, UserSettings
+from cms.models import Page, Placeholder
 from cms.models.pluginmodel import CMSPlugin
 from cms.plugin_base import CMSPluginBase
 from cms.plugin_pool import plugin_pool
@@ -43,10 +41,10 @@ from cms.test_utils.testcases import (
 )
 from cms.test_utils.util.fuzzy_int import FuzzyInt
 from cms.toolbar.toolbar import CMSToolbar
+from cms.toolbar.utils import get_toolbar_from_request
 from cms.utils.conf import get_cms_setting
 from cms.utils.copy_plugins import copy_plugins_to
-from cms.utils.i18n import force_language
-from cms.utils.plugins import get_plugins_for_page, get_plugins
+from cms.utils.plugins import get_plugins
 from django.utils.http import urlencode
 
 from djangocms_text_ckeditor.models import Text
@@ -57,6 +55,10 @@ from djangocms_text_ckeditor.utils import plugin_to_tag
 def register_plugins(*plugins):
     for plugin in plugins:
         plugin_pool.register_plugin(plugin)
+
+    # clear cached properties
+    plugin_pool._clear_cached()
+
     try:
         yield
     finally:
@@ -64,6 +66,13 @@ def register_plugins(*plugins):
             plugin_pool.unregister_plugin(plugin)
 
 
+def _render_placeholder(placeholder, context, **kwargs):
+    request = context['request']
+    toolbar = get_toolbar_from_request(request)
+    content_renderer = toolbar.content_renderer
+    return content_renderer.render_placeholder(placeholder, context, **kwargs)
+
+
 class DumbFixturePlugin(CMSPluginBase):
     model = CMSPlugin
     name = "Dumb Test Plugin. It does nothing."
@@ -90,7 +99,9 @@ plugin_pool.register_plugin(DumbFixturePluginWithUrls)
 
 
 class PluginsTestBaseCase(CMSTestCase):
+
     def setUp(self):
+        plugin_pool._clear_cached()
         self.super_user = self._create_user("test", True, True)
         self.slave = self._create_user("slave", True)
 
@@ -123,9 +134,9 @@ class PluginsTestBaseCase(CMSTestCase):
 
 
 class PluginsTestCase(PluginsTestBaseCase):
-    def _create_text_plugin_on_page(self, page):
+    def _create_text_plugin_on_page(self, page, slot='col_left'):
         plugin = api.add_plugin(
-            placeholder=page.placeholders.get(slot="body"),
+            placeholder=page.placeholders.get(slot=slot),
             plugin_type='TextPlugin',
             language=settings.LANGUAGES[0][0],
             body=''
@@ -153,7 +164,7 @@ class PluginsTestCase(PluginsTestBaseCase):
         # add a new text plugin
         page_data = self.get_new_page_data()
         self.client.post(URL_CMS_PAGE_ADD, page_data)
-        page = Page.objects.all()[0]
+        page = Page.objects.drafts().first()
         created_plugin_id = self._create_text_plugin_on_page(page)
         # now edit the plugin
         txt = self._edit_text_plugin(created_plugin_id, "Hello World")
@@ -165,7 +176,7 @@ class PluginsTestCase(PluginsTestBaseCase):
         placeholder = self.get_placeholder()
         url = URL_CMS_PLUGIN_ADD + '?' + urlencode({
             'plugin_type': "ArticlePlugin",
-            'plugin_language': settings.LANGUAGES[0][0],
+            'target_language': settings.LANGUAGES[0][0],
             'placeholder_id': placeholder.pk,
         })
         superuser = self.get_superuser()
@@ -228,7 +239,7 @@ class PluginsTestCase(PluginsTestBaseCase):
         with self.settings(CMS_PLACEHOLDER_CONF=CMS_PLACEHOLDER_CONF):
             page_data = self.get_new_page_data()
             self.client.post(URL_CMS_PAGE_ADD, page_data)
-            page = Page.objects.all()[0]
+            page = Page.objects.drafts().first()
             installed_plugins = plugin_pool.get_all_plugins('body', page)
             installed_plugins = [cls.__name__ for cls in installed_plugins]
             self.assertNotIn('TextPlugin', installed_plugins)
@@ -244,7 +255,7 @@ class PluginsTestCase(PluginsTestBaseCase):
         with self.settings(CMS_PLACEHOLDER_CONF=CMS_PLACEHOLDER_CONF):
             page_data = self.get_new_page_data()
             self.client.post(URL_CMS_PAGE_ADD, page_data)
-            page = Page.objects.all()[0]
+            page = Page.objects.drafts().first()
             installed_plugins = plugin_pool.get_all_plugins('body', page)
             installed_plugins = [cls.__name__ for cls in installed_plugins]
             self.assertNotIn('TextPlugin', installed_plugins)
@@ -253,18 +264,18 @@ class PluginsTestCase(PluginsTestBaseCase):
         page_data = self.get_new_page_data()
         response = self.client.post(URL_CMS_PAGE_ADD, page_data)
         self.assertEqual(response.status_code, 302)
-        page = Page.objects.all()[0]
+        page = Page.objects.drafts().first()
         response = self.client.post(URL_CMS_PAGE_PUBLISH % (page.pk, 'en'))
         self.assertEqual(response.status_code, 302)
         created_plugin_id = self._create_text_plugin_on_page(page)
-        page = Page.objects.all()[0]
+        page = Page.objects.drafts().first()
         self.assertEqual(page.is_dirty('en'), True)
         response = self.client.post(URL_CMS_PAGE_PUBLISH % (page.pk, 'en'))
         self.assertEqual(response.status_code, 302)
-        page = Page.objects.all()[0]
+        page = Page.objects.drafts().first()
         self.assertEqual(page.is_dirty('en'), False)
         self._edit_text_plugin(created_plugin_id, "Hello World")
-        page = Page.objects.all()[0]
+        page = Page.objects.drafts().first()
         self.assertEqual(page.is_dirty('en'), True)
 
     def test_plugin_order(self):
@@ -288,7 +299,8 @@ class PluginsTestCase(PluginsTestBaseCase):
             self.assertEqual(text_plugin_2.position, 1)
             self.assertEqual(db_plugin_2.position, 1)
             ## Finally we render the placeholder to test the actual content
-            rendered_placeholder = ph_en.render(self.get_context(page_en.get_absolute_url(), page=page_en), None)
+            context = self.get_context(page_en.get_absolute_url(), page=page_en)
+            rendered_placeholder = _render_placeholder(ph_en, context)
             self.assertEqual(rendered_placeholder, "I'm the firstI'm the second")
 
     def test_plugin_order_alt(self):
@@ -315,7 +327,7 @@ class PluginsTestCase(PluginsTestBaseCase):
             'placeholder_id': placeholder.id,
             'plugin_id': text_plugin_1.id,
             'plugin_parent': '',
-            'plugin_language': 'en',
+            'target_language': 'en',
             'plugin_order[]': [text_plugin_1.id, text_plugin_2.id, text_plugin_3.id],
         }
 
@@ -335,9 +347,11 @@ class PluginsTestCase(PluginsTestBaseCase):
             self.assertEqual(CMSPlugin.objects.get(pk=text_plugin_3.pk).position, 2)
 
             ## Finally we render the placeholder to test the actual content
-            rendered_placeholder = placeholder.render(self.get_context(draft_page.get_absolute_url(), page=draft_page), None)
+            draft_page_context = self.get_context(draft_page.get_absolute_url(), page=draft_page)
+            rendered_placeholder = _render_placeholder(placeholder, draft_page_context)
             self.assertEqual(rendered_placeholder, "I'm the firstI'm the secondI'm the third")
-            rendered_live_placeholder = live_placeholder.render(self.get_context(live_page.get_absolute_url(), page=live_page), None)
+            live_page_context = self.get_context(live_page.get_absolute_url(), page=live_page)
+            rendered_live_placeholder = _render_placeholder(live_placeholder, live_page_context)
             self.assertEqual(rendered_live_placeholder, "I'm the firstI'm the secondI'm the third")
 
         columns = api.add_plugin(placeholder, "MultiColumnPlugin", "en")
@@ -352,7 +366,7 @@ class PluginsTestCase(PluginsTestBaseCase):
             'placeholder_id': placeholder.id,
             'plugin_id': text_plugin_1.id,
             'plugin_parent': '',
-            'plugin_language': 'en',
+            'target_language': 'en',
             'plugin_order[]': [
                 text_plugin_1.id,
                 text_plugin_2.id,
@@ -652,7 +666,7 @@ class PluginsTestCase(PluginsTestBaseCase):
             template='nav_playground.html'
         )
         plugin = api.add_plugin(
-            placeholder=page.placeholders.get(slot="body"),
+            placeholder=page.placeholders.get(slot='body'),
             language='en',
             plugin_type='TextPlugin',
             body=''
@@ -734,7 +748,7 @@ class PluginsTestCase(PluginsTestBaseCase):
         ph.save()
         url = URL_CMS_PLUGIN_ADD + '?' + urlencode({
             'plugin_type': "TextPlugin",
-            'plugin_language': settings.LANGUAGES[0][0],
+            'target_language': settings.LANGUAGES[0][0],
             'placeholder': ph.pk,
             'plugin_parent': plugin.pk
 
@@ -744,7 +758,7 @@ class PluginsTestCase(PluginsTestBaseCase):
         self.assertEqual(response.status_code, 400)
 
     def test_register_plugin_twice_should_raise(self):
-        number_of_plugins_before = len(plugin_pool.get_all_plugins())
+        number_of_plugins_before = len(plugin_pool.registered_plugins)
         # The first time we register the plugin is should work
         with register_plugins(DumbFixturePlugin):
             # Let's add it a second time. We should catch and exception
@@ -755,11 +769,11 @@ class PluginsTestCase(PluginsTestBaseCase):
                 raised = True
             self.assertTrue(raised)
         # Let's make sure we have the same number of plugins as before:
-        number_of_plugins_after = len(plugin_pool.get_all_plugins())
+        number_of_plugins_after = len(plugin_pool.registered_plugins)
         self.assertEqual(number_of_plugins_before, number_of_plugins_after)
 
     def test_unregister_non_existing_plugin_should_raise(self):
-        number_of_plugins_before = len(plugin_pool.get_all_plugins())
+        number_of_plugins_before = len(plugin_pool.registered_plugins)
         raised = False
         try:
             # There should not be such a plugin registered if the others tests
@@ -769,7 +783,7 @@ class PluginsTestCase(PluginsTestBaseCase):
             raised = True
         self.assertTrue(raised)
         # Let's count, to make sure we didn't remove a plugin accidentally.
-        number_of_plugins_after = len(plugin_pool.get_all_plugins())
+        number_of_plugins_after = len(plugin_pool.registered_plugins)
         self.assertEqual(number_of_plugins_before, number_of_plugins_after)
 
     def test_search_pages(self):
@@ -807,10 +821,21 @@ class PluginsTestCase(PluginsTestBaseCase):
         plugin.add_root(instance=plugin)
 
         # this should not raise any errors, but just ignore the empty plugin
-        out = placeholder.render(self.get_context(), width=300)
+        out = _render_placeholder(placeholder, self.get_context(), width=300)
         self.assertFalse(len(out))
         self.assertFalse(len(placeholder._plugins_cache))
 
+    def test_repr(self):
+        non_saved_plugin = CMSPlugin()
+        self.assertIsNone(non_saved_plugin.pk)
+        self.assertIn('id=None', repr(non_saved_plugin))
+        self.assertIn("plugin_type=''", repr(non_saved_plugin))
+
+        saved_plugin = CMSPlugin.objects.create(plugin_type='TextPlugin')
+        self.assertIn('id={}'.format(saved_plugin.pk), repr(saved_plugin))
+        self.assertIn("plugin_type='{}'".format(saved_plugin.plugin_type), repr(saved_plugin))
+
+
     def test_pickle(self):
         page = api.create_page("page", "nav_playground.html", "en")
         placeholder = page.placeholders.get(slot='body')
@@ -892,7 +917,7 @@ class PluginsTestCase(PluginsTestBaseCase):
         title = page.get_title_obj('en')
         page.creation_date = one_day_ago
         page.changed_date = one_day_ago
-        plugin_id = self._create_text_plugin_on_page(page)
+        plugin_id = self._create_text_plugin_on_page(page, slot='body')
         plugin = self._edit_text_plugin(plugin_id, "fnord")
 
         actual_last_modification_time = CMSSitemap().lastmod(title)
@@ -920,6 +945,7 @@ class PluginsTestCase(PluginsTestBaseCase):
             post = {
                 'plugin_id': child_plugin.pk,
                 'placeholder_id': page.placeholders.get(slot='right-column').pk,
+                'target_language': 'en',
                 'plugin_parent': '',
             }
 
@@ -930,146 +956,6 @@ class PluginsTestCase(PluginsTestBaseCase):
             from cms.utils.plugins import build_plugin_tree
             build_plugin_tree(page.placeholders.get(slot='right-column').get_plugins_list())
 
-    def test_get_plugins_for_page(self):
-        page_en = api.create_page("PluginOrderPage", "col_two.html", "en",
-                                  slug="page1", published=True, in_navigation=True)
-        ph_en = page_en.placeholders.get(slot="col_left")
-        text_plugin_1 = api.add_plugin(ph_en, "TextPlugin", "en", body="I'm inside an existing placeholder.")
-        # This placeholder is not in the template.
-        ph_en_not_used = page_en.placeholders.create(slot="not_used")
-        text_plugin_2 = api.add_plugin(ph_en_not_used, "TextPlugin", "en", body="I'm inside a non-existent placeholder.")
-        page_plugins = get_plugins_for_page(None, page_en, page_en.get_title_obj_attribute('language'))
-        db_text_plugin_1 = page_plugins.get(pk=text_plugin_1.pk)
-        self.assertRaises(CMSPlugin.DoesNotExist, page_plugins.get, pk=text_plugin_2.pk)
-        self.assertEqual(db_text_plugin_1.pk, text_plugin_1.pk)
-
-    def test_plugin_move_with_reload(self):
-        action_options = {
-            PLUGIN_MOVE_ACTION: {
-                'requires_reload': True
-            },
-            PLUGIN_COPY_ACTION: {
-                'requires_reload': True
-            },
-        }
-        non_reload_action_options = {
-            PLUGIN_MOVE_ACTION: {
-                'requires_reload': False
-            },
-            PLUGIN_COPY_ACTION: {
-                'requires_reload': False
-            },
-        }
-        ReloadDrivenPlugin = type('ReloadDrivenPlugin', (CMSPluginBase,), dict(action_options=action_options, render_plugin=False))
-        NonReloadDrivenPlugin = type('NonReloadDrivenPlugin', (CMSPluginBase,), dict(action_options=non_reload_action_options, render_plugin=False))
-
-        with register_plugins(ReloadDrivenPlugin, NonReloadDrivenPlugin):
-            page = api.create_page("page", "nav_playground.html", "en", published=True)
-            source_placeholder = page.placeholders.get(slot='body')
-            target_placeholder = page.placeholders.get(slot='right-column')
-            plugin_1 = api.add_plugin(source_placeholder, ReloadDrivenPlugin, settings.LANGUAGES[0][0])
-            plugin_2 = api.add_plugin(source_placeholder, NonReloadDrivenPlugin, settings.LANGUAGES[0][0])
-
-            with force_language('en'):
-                plugin_1_action_urls = plugin_1.get_action_urls()
-
-            reload_expected = {
-                'reload': True,
-                'urls': plugin_1_action_urls,
-            }
-
-            # Test Plugin reload == True on Move
-            post = {
-                'plugin_id': plugin_1.pk,
-                'placeholder_id': target_placeholder.pk,
-                'plugin_parent': '',
-            }
-
-            endpoint = self.get_move_plugin_uri(plugin_1)
-
-            response = self.client.post(endpoint, post)
-            self.assertEqual(response.status_code, 200)
-            self.assertEqual(json.loads(response.content.decode('utf8')), reload_expected)
-
-            with force_language('en'):
-                plugin_2_action_urls = plugin_2.get_action_urls()
-
-            no_reload_expected = {
-                'reload': False,
-                'urls': plugin_2_action_urls,
-            }
-
-            # Test Plugin reload == False on Move
-            post = {
-                'plugin_id': plugin_2.pk,
-                'placeholder_id': target_placeholder.pk,
-                'plugin_parent': '',
-            }
-
-            response = self.client.post(endpoint, post)
-            self.assertEqual(response.status_code, 200)
-            self.assertEqual(json.loads(response.content.decode('utf8')), no_reload_expected)
-
-    def test_plugin_copy_with_reload(self):
-        action_options = {
-            PLUGIN_MOVE_ACTION: {
-                'requires_reload': True
-            },
-            PLUGIN_COPY_ACTION: {
-                'requires_reload': True
-            },
-        }
-        non_reload_action_options = {
-            PLUGIN_MOVE_ACTION: {
-                'requires_reload': False
-            },
-            PLUGIN_COPY_ACTION: {
-                'requires_reload': False
-            },
-        }
-        ReloadDrivenPlugin = type('ReloadDrivenPlugin', (CMSPluginBase,), dict(action_options=action_options, render_plugin=False))
-        NonReloadDrivenPlugin = type('NonReloadDrivenPlugin', (CMSPluginBase,), dict(action_options=non_reload_action_options, render_plugin=False))
-
-        endpoint = self.get_admin_url(Page, 'copy_plugins') + '?cms_path=/en/'
-
-        user_settings = UserSettings.objects.create(
-            language="en",
-            user=self.super_user,
-            clipboard=Placeholder.objects.create(slot='clipboard'),
-        )
-
-        with register_plugins(ReloadDrivenPlugin, NonReloadDrivenPlugin):
-            page = api.create_page("page", "nav_playground.html", "en", published=True)
-            source_placeholder = page.placeholders.get(slot='body')
-            target_placeholder = user_settings.clipboard
-            api.add_plugin(source_placeholder, ReloadDrivenPlugin, settings.LANGUAGES[0][0])
-            plugin_2 = api.add_plugin(source_placeholder, NonReloadDrivenPlugin, settings.LANGUAGES[0][0])
-
-            # Test Plugin reload == True on Copy
-            copy_data = {
-                'source_placeholder_id': source_placeholder.pk,
-                'target_placeholder_id': target_placeholder.pk,
-                'target_language': settings.LANGUAGES[0][0],
-                'source_language': settings.LANGUAGES[0][0],
-            }
-            response = self.client.post(endpoint, copy_data)
-            self.assertEqual(response.status_code, 200)
-            json_response = json.loads(response.content.decode('utf8'))
-            self.assertEqual(json_response['reload'], True)
-
-            # Test Plugin reload == False on Copy
-            copy_data = {
-                'source_placeholder_id': source_placeholder.pk,
-                'source_plugin_id': plugin_2.pk,
-                'target_placeholder_id': target_placeholder.pk,
-                'target_language': settings.LANGUAGES[0][0],
-                'source_language': settings.LANGUAGES[0][0],
-            }
-            response = self.client.post(endpoint, copy_data)
-            self.assertEqual(response.status_code, 200)
-            json_response = json.loads(response.content.decode('utf8'))
-            self.assertEqual(json_response['reload'], False)
-
     def test_custom_plugin_urls(self):
         plugin_url = urlresolvers.reverse('admin:dumbfixtureplugin')
 
@@ -1112,7 +998,7 @@ class PluginsTestCase(PluginsTestBaseCase):
         }
 
         toolbar_struct = get_toolbar_plugin_struct(
-            plugins=plugin_pool.get_all_plugins(),
+            plugins=plugin_pool.registered_plugins,
             slot=placeholder.slot,
             page=page,
         )
@@ -1142,17 +1028,9 @@ class PluginsTestCase(PluginsTestBaseCase):
 
         with self.login_user_context(staff_user):
             request = self.get_request(page_url, page=page)
-            request.session['cms_edit'] = True
             request.toolbar = CMSToolbar(request)
-            content_renderer = self.get_content_renderer(request=request)
-            context = {'request': request, 'cms_content_renderer': content_renderer}
-            output = content_renderer.render_placeholder(
-                placeholder,
-                context=SekizaiContext(context),
-                language='en',
-                page=page,
-                editable=True,
-            )
+            renderer = self.get_structure_renderer(request=request)
+            output = renderer.render_placeholder(placeholder, language='en', page=page)
             self.assertIn('<a data-rel="add" href="TextPlugin">Text</a>', output)
             self.assertNotIn('<a data-rel="add" href="LinkPlugin">Link</a>', output)
 
@@ -1242,7 +1120,7 @@ class PluginsTestCase(PluginsTestBaseCase):
         # add a new text plugin
         page_data = self.get_new_page_data()
         self.client.post(URL_CMS_PAGE_ADD, page_data)
-        page = Page.objects.all()[0]
+        page = Page.objects.drafts().first()
         created_plugin_id = self._create_text_plugin_on_page(page)
 
         # now edit the plugin
@@ -1303,22 +1181,26 @@ class PluginManyToManyTestCase(PluginsTestBaseCase):
         context = self.get_context(path=page_en.get_absolute_url())
         request = context['request']
         plugins = get_plugins(request, ph_en, page_en.template)
+        content_renderer = self.get_content_renderer()
+
         for plugin in plugins:
             if plugin.title == 'custom template':
+                content = content_renderer.render_plugin(plugin, context, ph_en)
                 self.assertEqual(plugin.get_plugin_class_instance().get_render_template({}, plugin, ph_en), 'articles_custom.html')
-                self.assertTrue('Articles Custom template' in plugin.render_plugin(context, ph_en))
+                self.assertTrue('Articles Custom template' in content)
             else:
+                content = content_renderer.render_plugin(plugin, context, ph_en)
                 self.assertEqual(plugin.get_plugin_class_instance().get_render_template({}, plugin, ph_en), 'articles.html')
-                self.assertFalse('Articles Custom template' in plugin.render_plugin(context, ph_en))
+                self.assertFalse('Articles Custom template' in content)
 
     def test_add_plugin_with_m2m(self):
         # add a new text plugin
         self.assertEqual(ArticlePluginModel.objects.count(), 0)
         page_data = self.get_new_page_data()
         self.client.post(URL_CMS_PAGE_ADD, page_data)
-        page = Page.objects.all()[0]
+        page = Page.objects.drafts().first()
         page.publish('en')
-        placeholder = page.placeholders.get(slot="body")
+        placeholder = page.placeholders.get(slot='col_left')
         add_url = self.get_add_plugin_uri(
             placeholder=placeholder,
             plugin_type='ArticlePlugin',
@@ -1342,8 +1224,8 @@ class PluginManyToManyTestCase(PluginsTestBaseCase):
         page_data = self.get_new_page_data()
         response = self.client.post(URL_CMS_PAGE_ADD, page_data)
         self.assertEqual(response.status_code, 302)
-        page = Page.objects.all()[0]
-        placeholder = page.placeholders.get(slot="body")
+        page = Page.objects.drafts().first()
+        placeholder = page.placeholders.get(slot='col_left')
 
         # add a plugin
         data = {
@@ -1432,7 +1314,7 @@ class PluginManyToManyTestCase(PluginsTestBaseCase):
 
         response = self.client.post(endpoint, copy_data)
         self.assertEqual(response.status_code, 200)
-        self.assertEqual(response.content.decode('utf8').count('"position":'), 1)
+        self.assertEqual(response.content.decode('utf8').count('"plugin_type": "ArticlePlugin"'), 1)
         # assert copy success
         self.assertEqual(CMSPlugin.objects.filter(language=self.FIRST_LANG).count(), 1)
         self.assertEqual(CMSPlugin.objects.filter(language=self.SECOND_LANG).count(), 1)
@@ -1636,7 +1518,7 @@ class MTIPluginsTestCase(PluginsTestBaseCase):
 
         # Create a page
         page = create_page("Test", "nav_playground.html", settings.LANGUAGES[0][0])
-        placeholder = page.placeholders.get(slot="body")
+        placeholder = page.placeholders.get(slot='body')
 
         # Add the MTI plugin
         add_url = self.get_add_plugin_uri(
@@ -1688,26 +1570,3 @@ class MTIPluginsTestCase(PluginsTestBaseCase):
                          'mti_pluginapp_lessmixedplugin')
         # Non plugins are skipped
         self.assertFalse(hasattr(NonPluginModel, 'cmsplugin_ptr'))
-
-
-class UserInputValidationPluginTest(PluginsTestBaseCase):
-
-    def test_error_response_escapes(self):
-        superuser = self.get_superuser()
-        page = api.create_page(
-            title='error page',
-            template='nav_playground.html',
-            language='en'
-        )
-        url = URL_CMS_PLUGIN_ADD + '?' + urlencode({
-            'plugin_type': 'TextPlugin"><script>alert("hello world")</script>',
-            'placeholder_id': page.placeholders.get(slot='body').id,
-            'cms_path': page.get_path(),
-            'plugin_language': settings.LANGUAGES[0][0],
-        })
-
-        with self.login_user_context(superuser):
-            response = self.client.get(url)
-
-        self.assertEqual(response.status_code, 400)
-        self.assertIn('TextPlugin&quot;&gt;&lt;script&gt;alert(&quot;hello world&quot;)&lt;/script&gt;', response.content.decode("utf-8"))
