@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
-from __future__ import with_statement
-from contextlib import contextmanager
-import inspect
-import warnings
-from cms.models.titlemodels import Title
-from cms.utils import get_language_from_request
-from cms.utils.compat import DJANGO_1_6
-from cms.utils.i18n import force_language, hide_untranslated
 from django.conf import settings
 from django.core.urlresolvers import NoReverseMatch, reverse, resolve
-from django.utils import six
+
+from cms.utils import get_current_site, get_language_from_request
+from cms.utils.i18n import (
+    force_language,
+    get_default_language_for_site,
+    get_fallback_languages,
+    hide_untranslated,
+    is_valid_site_language,
+)
 
 
 def mark_descendants(nodes):
@@ -22,18 +22,15 @@ def cut_levels(nodes, level):
     """
     For cutting the nav_extender levels if you have a from_level in the navigation.
     """
-    result = []
     if nodes:
         if nodes[0].level == level:
             return nodes
-    for node in nodes:
-        result += cut_levels(node.children, level)
-    return result
+    return sum((cut_levels(node.children, level) for node in nodes), [])
 
 
 def find_selected(nodes):
     """
-    Finds a selected nav_extender node 
+    Finds a selected nav_extender node
     """
     for node in nodes:
         if hasattr(node, "selected"):
@@ -46,15 +43,15 @@ def find_selected(nodes):
 
 def set_language_changer(request, func):
     """
-    
+
     Sets a language chooser function that accepts one parameter: language
     The function should return a url in the supplied language
     normally you would want to give it the get_absolute_url function with an optional language parameter
     example:
-    
+
     def get_absolute_url(self, language=None):
         reverse('product_view', args=[self.get_slug(language=language)])
-        
+
     Use this function in your nav extender views that have i18n slugs.
     """
     request._language_changer = func
@@ -63,9 +60,9 @@ def set_language_changer(request, func):
 def language_changer_decorator(language_changer):
     """
     A decorator wrapper for set_language_changer.
-    
+
         from menus.utils import language_changer_decorator
-        
+
         @language_changer_decorator(function_get_language_changer_url)
         def my_view_function(request, somearg):
             pass
@@ -100,20 +97,43 @@ class DefaultLanguageChanger(object):
 
     def get_page_path(self, lang):
         page = getattr(self.request, 'current_page', None)
-        if page:
-            with force_language(lang):
-                try:
-                    return page.get_absolute_url(language=lang, fallback=False)
-                except (Title.DoesNotExist, NoReverseMatch):
-                    if hide_untranslated(lang) and settings.USE_I18N:
-                        return '/%s/' % lang
-                    else:
-                        return page.get_absolute_url(language=lang, fallback=True)
+
+        if not page:
+            return '/%s/' % lang if settings.USE_I18N else '/'
+
+        page_languages = page.get_published_languages()
+
+        if lang in page_languages:
+            return page.get_absolute_url(lang, fallback=False)
+
+        site = get_current_site()
+
+        if is_valid_site_language(lang, site_id=site.pk):
+            _valid_language = True
+            _hide_untranslated = hide_untranslated(lang, site.pk)
         else:
-            if settings.USE_I18N:
-                return '/%s/' % lang
-            else:
-                return "/"
+            _valid_language = False
+            _hide_untranslated = False
+
+        if _hide_untranslated and settings.USE_I18N:
+            return '/%s/' % lang
+
+        default_language = get_default_language_for_site(site.pk)
+
+        if not _valid_language and default_language in page_languages:
+            # The request language is not configured for the current site.
+            # Fallback to the default language configured for the current site.
+            return page.get_absolute_url(default_language, fallback=False)
+
+        if _valid_language:
+            fallbacks = get_fallback_languages(lang, site_id=site.pk) or []
+            fallbacks = [_lang for _lang in fallbacks if _lang in page_languages]
+        else:
+            fallbacks = []
+
+        if fallbacks:
+            return page.get_absolute_url(fallbacks[0], fallback=False)
+        return '/%s/' % lang if settings.USE_I18N else '/'
 
     def __call__(self, lang):
         page_language = get_language_from_request(self.request)
@@ -134,65 +154,10 @@ class DefaultLanguageChanger(object):
                 view_name = "%s:%s" % (view.namespace, view_name)
             url = None
             with force_language(lang):
-                with static_stringifier(view):  # This is a fix for Django < 1.7
-                    try:
-                        url = reverse(view_name, args=view.args, kwargs=view.kwargs, current_app=view.app_name)
-                    except NoReverseMatch:
-                        pass
+                try:
+                    url = reverse(view_name, args=view.args, kwargs=view.kwargs, current_app=view.app_name)
+                except NoReverseMatch:
+                    pass
             if url:
                 return url
         return '%s%s' % (self.get_page_path(lang), self.app_path)
-
-
-def simple_language_changer(func):
-    warnings.warn("simple_language_changer is deprecated and will be removed in "
-        "2.5! This is the default behaviour now for non CMS managed views and is no longer needed.",
-        DeprecationWarning)
-
-    def _wrapped(request, *args, **kwargs):
-        set_language_changer(request, DefaultLanguageChanger(request))
-        return func(request, *args, **kwargs)
-    _wrapped.__name__ = func.__name__
-    _wrapped.__doc__ = func.__doc__
-    return _wrapped
-
-
-@contextmanager
-def static_stringifier(view):
-    """
-    In Django < 1.7 reverse tries to convert to string the arguments without
-    checking whether they are classes or instances.
-
-    This context manager monkeypatches the __unicode__ method of each view
-    argument if it's a class definition to render it a static method.
-    Before leaving we undo the monkeypatching.
-    """
-    if DJANGO_1_6:
-        for idx, arg in enumerate(view.args):
-            if inspect.isclass(arg):
-                if hasattr(arg, '__unicode__'):
-                    @staticmethod
-                    def custom_str():
-                        return six.text_type(arg)
-                    arg._original = arg.__unicode__
-                    arg.__unicode__ = custom_str
-                view.args[idx] = arg
-        for key, arg in view.kwargs.items():
-            if inspect.isclass(arg):
-                if hasattr(arg, '__unicode__'):
-                    @staticmethod
-                    def custom_str():
-                        return six.text_type(arg)
-                    arg._original = arg.__unicode__
-                    arg.__unicode__ = custom_str
-                view.kwargs[key] = arg
-    yield
-    if DJANGO_1_6:
-        for idx, arg in enumerate(view.args):
-            if inspect.isclass(arg):
-                if hasattr(arg, '__unicode__'):
-                    arg.__unicode__ = arg._original
-        for key, arg in view.kwargs.items():
-            if inspect.isclass(arg):
-                if hasattr(arg, '__unicode__'):
-                    arg.__unicode__ = arg._original
\ No newline at end of file
